/*
Copyright (c) 2021 nxa07283, NXP
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    (1) Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer. 

    (2) Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.  
    
    (3)The name of the author may not be used to
    endorse or promote products derived from this software without
    specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE. 
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <fcntl.h>
#include <linux/i2c-dev.h>
#include <linux/i2c.h>
#include <sys/ioctl.h>

#include <ntag5.h>

/* in units of 10ms */
#define TIMEOUT (100)

int i2c_open(ntag_t ntag, struct ntag5_hw_config *hw_config) {
	
	int file;
	int res;
	
	char filename[32];
	snprintf(filename, sizeof(filename), "/dev/i2c-%d", hw_config->i2c_adapter);
	file = open(filename, O_RDWR);
	if(file < 0) {
		return -1;
	};	
	
	res = ioctl(file, I2C_SLAVE, hw_config->i2c_address);
	if(res < 0) {
		close(file);
		return -1;
	};

	res = ioctl(file, I2C_TIMEOUT, TIMEOUT);
	if(res < 0) {
		close(file);
		return -1;
	};
	
	ntag->i2c_file = file;
	ntag->i2c_address = hw_config->i2c_address;

	return 0;
	
}


int i2c_close(ntag_t ntag) {
	
	int file = ntag->i2c_file;
	if(file) {
		ntag->i2c_file = 0;
		return close(file);
	} else {
		errno = -ENXIO;
		return -1;
	}
}


int i2c_rdwr(ntag_t ntag, struct i2c_rdwr_ioctl_data *msgset)
{
	for(int i = 0; i < msgset->nmsgs; i++) {
		msgset->msgs[i].addr = ntag->i2c_address;
	};	
	return ioctl(ntag->i2c_file, I2C_RDWR, msgset);
}
